jsx3.util.Logger.getLogger("org.isocat").info("Loading class[org.isocat.gui.Explorer]");

/**
 * The Explorer-like view
 * @author Menzo.Windhouwer@mpi.nl
 */
jsx3.lang.Class.defineClass("org.isocat.gui.Explorer", null, [], function(Explorer, Explorer_prototype) {
	
	/**
	 * The class logger
	 */
	Explorer.log = jsx3.util.Logger.getLogger("org.isocat.gui.Explorer");
	
	/**
	 * The root block of the view
	 */
	Explorer_prototype.view;
	
	/**
	 * The workspace view
	 */
	Explorer_prototype.ws;
	
	/**
	 * The tabs
	 */
	Explorer_prototype.tabs;
	
	/**
	 * The overview tab
	 */
	Explorer_prototype.overview;
	
	/**
	 * The controller of the view loaded in the overview tab
	 */
	Explorer_prototype.overviewView;
	
	/**
	 * The detail tab
	 */
	Explorer_prototype.detail;
	
	/**
	 * The DCS viewer
	 */
	Explorer_prototype.dcs;
	
	/**
	 * The DC viewer
	 */
	Explorer_prototype.dc;
	
	/**
	 * Constructor
	 */
	Explorer_prototype.init = function(view) {
		org.isocat.gui.Explorer.log.info("initialize explorer view["+this+"]");

		// lookup all GUI components
		this.view = view;
		org.isocat.gui.Explorer.log.info("- explorer view["+this.view+"]");
		var v     = this.view.getDescendantOfName("WorkspaceViewer");
		this.ws   = v.controller;
		org.isocat.gui.Explorer.log.info("- explorer WS view["+v+"]["+this.ws+"]");
		this.tabs = this.view.getDescendantOfName("tabs");
		org.isocat.gui.Explorer.log.info("- explorer overview["+this.tabs+"]");
		this.overview = this.view.getDescendantOfName("overview");
		org.isocat.gui.Explorer.log.info("- explorer overview["+this.overview+"]");
		this.overviewView = null;
		this.detail = this.view.getDescendantOfName("detail");
		org.isocat.gui.Explorer.log.info("- explorer detail["+this.detail+"]");
		this.dcs = this.view.getDescendantOfName("ExplorerDataCategorySelectionViewer");
		org.isocat.gui.Explorer.log.info("- explorer DCS viewer["+this.dcs+"]");
		this.dc = this.view.getDescendantOfName("ExplorerDataCategoryViewer");
		org.isocat.gui.Explorer.log.info("- explorer DC viewer["+this.dc+"]");

		org.isocat.gui.Explorer.log.info("initialized explorer view["+this+"]");
		
		this.view.controller = this;

		// create message
		var message = {
			jss: "org.pagebus.msg.Message",
			jssv: "1.0.0",
			header: { },
			body: this
		};
		
		// publish the message using PageBus
		window.PageBus.publish("org.isocat.gui.Explorer.init", message);
	};
	
	/**
	 * Reset the view
	 */
	Explorer_prototype.reset = function() {
		this.showWorkspace(org.isocat.profile);
		this.loadDataCategorySelectionView().reset();
		this.loadDataCategoryView().reset();
		this.showUser(org.isocat.profile);
        };
        
        /**
         * Load a view in the overview tab
         */
        Explorer_prototype.loadOverview = function(component) {
        	if (this.tabs.getSelectedIndex()!=0)
        		this.tabs.setSelectedIndex(0,true);
        	else
        		this.closeOverview();
		this.overview.removeChildren();
		this.overviewView = this.overview.loadAndCache(component,true).controller;
		return this.overviewView;
        };
        
        /**
         * Close the view in the overview tab
         */
        Explorer_prototype.closeOverview = function() {
        	if (this.overviewView != null)
        		if (typeof(this.overviewView.close)=='function')
        			this.overviewView.close();
        };

	/**
	 * Show a specific user's WS
	 */
        Explorer_prototype.showWorkspace = function(profile) {
		org.isocat.gui.Explorer.log.info("show the selected WS["+profile.user()+"] in the explorer view");

		//load the user's WS in the WS viewer
		this.ws.showWorkspace(profile);

		org.isocat.gui.Explorer.log.info("showing the selected WS["+profile.user()+"] in the explorer view?");
        };
	
	/**
	 * Show user overview
	 */
        Explorer_prototype.showUser = function(profile) {
		org.isocat.gui.Explorer.log.info("show the selected user["+profile.user()+"] in the explorer view");

		var userOverview = this.loadOverview("components/User.xml");
		userOverview.showUser(profile);
        };

	/**
	 * Show DCRB overview
	 */
        Explorer_prototype.showDCRB = function(dcrb) {
		org.isocat.gui.Explorer.log.info("show the DCRB in the explorer view");

		if (dcrb.mode.indexOf('u') >= 0) {
			var editor = this.loadOverview("components/GroupEditor.xml");
			editor.editGroup(new org.isocat.data.Group(0,"dcrb"),dcrb.mode);
		} else {
			var overview = this.loadOverview("components/DCRB.xml");
			overview.showDCRB(dcrb);
		}
	}
	
	/**
	 * Show TDG overview
	 */
        Explorer_prototype.showTDG = function(tdg) {
		org.isocat.gui.Explorer.log.info("show the selected TDG["+tdg.id+"]["+tdg.name+"] in the explorer view");

		if (tdg.mode.indexOf('u') >= 0) {
			var groupEditor = this.loadOverview("components/GroupEditor.xml");
			groupEditor.editGroup(new org.isocat.data.Group(tdg.id,"tdg"),tdg.mode);
		} else {
			var tdgOverview = this.loadOverview("components/TDG.xml");
			tdgOverview.showTDG(tdg);
		}
	}
	
	/**
	 * Show Group
	 */
    Explorer_prototype.showGroup = function(group) {
		org.isocat.gui.Explorer.log.info("show the selected Group["+group.id+"]["+group.name+"] in the explorer view");

		if (group.mode.indexOf('u') >= 0) {
			var groupEditor = this.loadOverview("components/GroupEditor.xml");
			groupEditor.editGroup(new org.isocat.data.Group(group.id),group.mode);
		} else {
			var groupOverview = this.loadOverview("components/Group.xml");
			groupOverview.showGroup(group);
		}
	}

    /**
	 * Show a specific Group in a seperate window
	 */
    Explorer_prototype.showGroupWindow = function(grp) {
		org.isocat.gui.Explorer.log.info("show the selected Group["+grp.id+"] in the explorer view in a seperate window");

		var name = "window-grp-" + grp.id;
		var title = "Group: " + grp.name;

		var window = org.isocat.desktop.getChild(name);
		if (typeof(window) == 'undefined') {
			window = org.isocat.createWindow(name,title);
			var view = window.loadAndCache("components/Group.xml",true);
			view.controller.showGroup(grp);
			view.controller.setWindow(window);
			
			org.isocat.gui.Explorer.log.info("loaded new Group["+view+"]");
		} else {
			org.isocat.gui.Explorer.log.info("window["+name+"]["+title+"] already exists!");
			window.focus();
		}
    };
	
	/**
	 * Show Change Request
	 */
    Explorer_prototype.showChangeRequest = function(cr) {
		org.isocat.gui.Explorer.log.info("show the selected CR["+cr.id+"]["+cr.name+"]["+cr.mode+"] in the explorer view");

        if (cr.mode.indexOf('u') >= 0) {
			var editor = this.loadOverview("components/ChangeRequestEditor.xml");
			editor.editChangeRequest(cr,cr.mode);
		} else {
			var overview = this.loadOverview("components/ChangeRequest.xml");
			overview.showChangeRequest(cr);
		}
	}
	
	/**
	 * New Group
	 */
    Explorer_prototype.newGroup = function() {
		org.isocat.gui.Explorer.log.info("show the GroupEditor for a new group in the explorer view");

		var groupEditor = this.loadOverview("components/GroupEditor.xml");
		groupEditor.newGroup();
	}
	
	/**
	 * Load a DCS view
	 */
    Explorer_prototype.loadDataCategorySelectionView = function() {
		var view = this.dcs.getChild("DataCategorySelectionViewer");
		if (view != null) {
			org.isocat.gui.Explorer.log.info("existing DataCategorySelectionViewer["+view+"]");
			return view.controller;
		}
		view = this.dcs.loadAndCache("components/DataCategorySelectionViewer.xml",true);
		org.isocat.gui.Explorer.log.info("new DataCategorySelectionViewer["+view+"]");
		return view.controller;
    };
	 
	/**
	 * Show a specific DCS
	 */
    Explorer_prototype.showDataCategorySelection = function(dcs) {
		org.isocat.gui.Explorer.log.info("show the selected DCS["+dcs.type()+","+dcs.key()+"] in the explorer DCS view["+this.dcs+"]");

		if (this.tabs.getSelectedIndex()!=1) {
			this.closeOverview();
			this.tabs.setSelectedIndex(1,true);
		}

		var DCSViewer = this.loadDataCategorySelectionView();
		DCSViewer.showDataCategorySelection(dcs);
    };
	
	/**
	 * Load a DC view
	 */
    Explorer_prototype.loadDataCategoryView = function() {
		// first repaint to make sure we don't find leftovers ...
		// THIS MAY STILL BE NEEDED, BUT INTERFERES WITH MULTIPLE SELECTS: this.view.repaint();

		var view = this.dc.getChild("DataCategoryViewer");
		if (view != null) {
			org.isocat.gui.Explorer.log.info("existing DataCategoryViewer["+view+"]");
			return view.controller;
		}
		view = this.dc.loadAndCache("components/DataCategoryViewer.xml",true);
		org.isocat.gui.Explorer.log.info("loaded new DataCategoryViewer["+view+"]");
		return view.controller;
    };
	
	/**
	 * Show a specific DC
	 */
    Explorer_prototype.showDataCategory = function(dc,dcs) {
		org.isocat.gui.Explorer.log.info("show the selected DC["+dc.id+"] in DCS["+dcs+"] in the explorer view");

		var DCViewer = this.loadDataCategoryView();
		DCViewer.showDataCategory(dc,dcs);

		this.tabs.setSelectedIndex(1,true);
    };
	
	/**
	 * Show a specific DC in a seperate window
	 */
    Explorer_prototype.showDataCategoryWindow = function(dc,dcs) {
		org.isocat.gui.Explorer.log.info("show the selected DC["+dc.id+"] in the explorer view is a seperate window");

		var name = "window-DC-" + dc.id;
		var title = "DC: " + dc.name + " - " + dc.version;

		var window = org.isocat.desktop.getChild(name);
		if (typeof(window) == 'undefined') {
			window = org.isocat.createWindow(name,title);
			var view = window.loadAndCache("components/DataCategoryViewer.xml",true);
			view.controller.showDataCategory(dc,dcs);
			view.controller.setWindow(window);
			
			org.isocat.gui.Explorer.log.info("loaded new DataCategoryViewer["+view+"]");
		} else {
			org.isocat.gui.Explorer.log.info("window["+name+"]["+title+"] already exists!");
			window.focus();
		}
    };
	
	/**
	 * Repaint this view
	 */
	Explorer_prototype.repaint = function() {
		this.view.repaint();
	};
	
	/**
	 * Deselect the DCS in the WS
	 */
	Explorer_prototype.deselectDCS = function() {
		this.ws.deselect();
	};

	/**
	 * Deselect the DC in the DCS
	 */
	Explorer_prototype.deselectDC = function() {
		this.dcs.getChild("DataCategorySelectionViewer").controller.deselect();
	};

	/**
	 * Detach the DCS view
	 */
	Explorer_prototype.detachDCS = function(view) {
		var dcs = view.controller.dcs;
		var name = "window-DCS-"+dcs.type()+"-"+dcs.key();
		var title = "DCS: "+dcs.name();

		org.isocat.gui.Explorer.log.info("detach DCS to window["+name+"]["+title+"] -> ["+org.isocat.desktop.getChild(name)+"]");
		
		var window = org.isocat.desktop.getChild(name);
		if ( typeof(window) == 'undefined') {
			window = org.isocat.createWindow(name,title);
			window.adoptChild(view,true,true);
			view.controller.setWindow(window);
			
			this.loadDataCategorySelectionView();

			this.deselectDCS();
		} else {
			org.isocat.gui.Explorer.log.info("window["+name+"]["+title+"] already exists!");
			window.focus();
		}
	};

	/**
	 * Detach the DC view
	 */
	Explorer_prototype.detachDC = function(view) {
		var dc = view.controller.dc;
		var name = "window-DC-" + dc.id;
		var title = "DC: " + dc.name + " - " + dc.version;

		org.isocat.gui.Explorer.log.info("detach DC to window["+name+"]["+title+"] -> ["+org.isocat.desktop.getChild(name)+"]");
		
		var window = org.isocat.desktop.getChild(name);
		if ( typeof(window) == 'undefined') {
			window = org.isocat.createWindow(name,title);
			window.adoptChild(view,true,true);
			view.controller.setWindow(window);// close some sections
			
			view.controller.resetView();
			
			this.loadDataCategoryView();

			this.deselectDC();
		} else {
			org.isocat.gui.Explorer.log.info("window["+name+"]["+title+"] already exists!");
			window.focus();
		}
	};
});
